#!/usr/bin/python3
import sys, os, pwd, grp, signal, time
from resource_management import *
from subprocess import call
from airflow_setup import *
import subprocess

class AirflowWorker(Script):
    """
    Contains the interface definitions for methods like install, 
    start, stop, status, etc. for the Airflow Server
    """
    def install(self, env):
        import params
        env.set_params(params)
        self.install_packages(env)
        Logger.info(format("Installing Airflow Service"))
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} pip"), ignore_failures=True)
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} setuptools"), ignore_failures=True)
        Execute(format("python3 -m pip install --upgrade {airflow_pip_params} docutils pytest-runner Cython"), ignore_failures=True)
        airflow_make_startup_script(env)
        airflow_make_systemd_scripts_worker(env)
        airflow_make_systemd_scripts_flower(env)
        airflow_make_systemd_scripts_kerberos(env)
        Execute(format("chmod 755 /usr/local/bin/airflow"))
        Execute(format("useradd {airflow_user}"), ignore_failures=True)
        Execute(format("mkdir -p {airflow_home}"))
        if not os.path.exists(params.base_log_folder):
            Execute(format("mkdir -p {base_log_folder}"))
            Execute(format("chown -R {airflow_user}:{airflow_group} {base_log_folder}"))
        airflow_make_startup_script(env)
        Execute(format("mkdir -p {airflow_home}/dags"))
        Execute(format("chown -R {airflow_user}:{airflow_group} {airflow_home}"))
        Execute(format("chmod 755 {airflow_home}"))
        Execute(format("chmod 755 {base_log_folder}"))
        Execute(format("chown -R {airflow_user}:{airflow_group} /var/run/airflow"))
        Execute(format("chmod 755 /var/run/airflow"))
        self.configure(env)
        
    def configure(self, env):
        import params
        env.set_params(params)
        airflow_configure(env)
        airflow_make_systemd_scripts_worker(env)
        
    def start(self, env):
        import params
        self.configure(env)
        
        Execute(format("chown -R {airflow_user}:{airflow_group} /var/run/airflow"))
        Execute(format("chmod 755 /var/run/airflow"))

        Execute(format("chown -R {airflow_user}:{airflow_group} {base_log_folder}"))
        Execute(format("chmod 755 {base_log_folder}"))
        
        Execute(format("sudo systemctl daemon-reload"))

        Execute(format("sudo systemctl start airflow-worker"))
        Execute(format("sudo systemctl enable airflow-worker")) 

        #Start Flower UI
        Execute(format("sudo systemctl start airflow-flower"))
        Execute(format("sudo systemctl enable airflow-flower")) 

        # Capture the process ID (PID) of the Airflow worker and write it to a new PID file
        Execute('ps -ef | grep "airflow celery worker" | grep "celery worker" | grep -v grep | awk \'{print $2}\' | tail -n 1 > ' + params.airflow_worker_pid_file, user=params.airflow_user)

        # Set ownership and permissions for the worker PID file
        Execute(format("sudo chown {airflow_user}:{airflow_group} {airflow_worker_pid_file}"))
        Execute(format("sudo chmod 644 {airflow_worker_pid_file}"))
        
        # Start Kerberos 
        if not params.kerberos_disable and params.kerberos_security_enabled:
              Execute(format("sudo systemctl daemon-reload"))
              Execute(format("sudo systemctl start airflow-kerberos"))
              Execute(format("sudo systemctl enable airflow-kerberos"))

    def stop(self, env):
        import params
        env.set_params(params)
        Execute(format("chown -R {airflow_user}:{airflow_group} /var/run/airflow"))
        Execute(format("chmod 755 /var/run/airflow"))
        
        Execute(format("sudo systemctl stop airflow-worker"))
        Execute(format("sudo systemctl disable airflow-worker"))

        #Stop Flower UI
        Execute(format("sudo systemctl stop airflow-flower"))
        Execute(format("sudo systemctl disable airflow-flower")) 

        Execute(format("while sudo systemctl is-active airflow-worker; do sleep 1; done"))

        File(params.airflow_worker_pid_file, action="delete")
        
        if params.kerberos_disable and params.kerberos_security_enabled:
              Execute(format("sudo systemctl stop airflow-kerberos"))
              Execute(format("sudo systemctl disable airflow-kerberos"))
              Execute(format("sudo systemctl daemon-reload"))
    
    def restart(self, env):
        self.stop(env)
        time.sleep(5)
        self.start(env)

    def status(self, env):
        import status_params
        env.set_params(status_params)
        #use built-in method to check status using pidfile
        check_process_status(status_params.airflow_worker_pid_file)

if __name__ == "__main__":
    AirflowWorker().execute()
